#ifdef WIN32
	#pragma comment(lib, "Ws2_32.lib")
	#define WIN32_LEAN_AND_MEAN
	#include <winsock.h>

#endif

#ifdef __linux__
	#include <socket.h>
#endif

#include <Endianness.hpp>

namespace Endianness
{
	template <>
	u8 ReverseByteOrder<unsigned char>(u8 val)
	{ return val;}

	template <>
	s8 ReverseByteOrder<signed char>(s8 val)
	{return val;}

	template <>
	u16 ReverseByteOrder<unsigned short>(u16 val)
	{
#if defined(_MSC_VER)
		return _byteswap_ushort(val);
#elif defined(__GNUC__)
		return __builtin_bswap16(val);
#endif
		return (val << 8) | (val >> 8);
	}

	template <>
	u32 ReverseByteOrder<unsigned>(u32 val)
	{
#ifdef _MSC_VER
		return _byteswap_ulong(val);
#endif

#ifdef __GNUC__
		return __builtin_bswap32(val);
#endif

		val = ((val << 8) & 0xFF00FF00 ) | ((val >> 8) & 0xFF00FF );
		return (val << 16) | (val >> 16);
	}

	template <>
	u64 ReverseByteOrder<unsigned long long>(u64 val)
	{
#if defined(_MSC_VER)
		return _byteswap_uint64(val);
#elif defined(__GNUC__)
		return __builtin_bswap64(val);
#else
		val = ((val << 8) & 0xFF00FF00FF00FF00ULL ) | ((val >> 8) & 0x00FF00FF00FF00FFULL );
		val = ((val << 16) & 0xFFFF0000FFFF0000ULL ) | ((val >> 16) & 0x0000FFFF0000FFFFULL );
		return (val << 32) | (val >> 32);
#endif
	}

	template <>
	s16 ReverseByteOrder<short>(s16 val)
	{
		return (val << 8) | ((val >> 8) & 0xFF);
	}

	template <>
	s32 ReverseByteOrder<int>(s32 val)
	{
		val = ((val << 8) & 0xFF00FF00) | ((val >> 8) & 0xFF00FF );
		return (val << 16) | ((val >> 16) & 0xFFFF);
	}

	template <>
	s64 ReverseByteOrder<long long>(s64 val)
	{
		val = ((val << 8) & 0xFF00FF00FF00FF00ULL ) | ((val >> 8) & 0x00FF00FF00FF00FFULL );
		val = ((val << 16) & 0xFFFF0000FFFF0000ULL ) | ((val >> 16) & 0x0000FFFF0000FFFFULL );
		return (val << 32) | ((val >> 32) & 0xFFFFFFFFULL);
	}


	float HostToNetworkOrder(float host) {
		if (IsLittleEndian())
			return ReverseByteOrder(host);
		else
			return host;
	}

	float NetworkToHostOrder(float network) {
		if (IsLittleEndian())
			return ReverseByteOrder(network);
		else
			return network;
	}

	double HostToNetworkOrder(double host) {
		if (IsLittleEndian())
			return ReverseByteOrder(host);
		else
			return host;
	}

	double NetworkToHostOrder(double network) {
		if (IsLittleEndian())
			return ReverseByteOrder(network);
		else
			return network;
	}
}